import { ICodeMirror } from '@jupyterlab/codemirror';
import { defineMultiplexingMode } from './codemirror-my-multiplex';
import { defineFinesse2Mode } from './codemirror-finesse2';
import { defineFinesse3Mode } from './codemirror-finesse3';

export function setupFinesseCodeMirror(codemirror: ICodeMirror): void {
  const cm = codemirror.CodeMirror;
  defineMultiplexingMode(codemirror);
  defineFinesse2Mode(codemirror);
  defineFinesse3Mode(codemirror);

  cm.defineMode('finesse-python', (config: any) => {
    const pmode = cm.getMode(config, 'python');
    return cm.myMultiplexingMode(
      pmode,
      {
        open: /(?=#kat2)/,
        close: /(?=""")/, // Match string end without consuming it
        mode: cm.getMode(config, 'text/x-finesse2'),
        delimStyle: 'delim',
      },
      {
        open: /(?=#kat)/,
        close: /(?=""")/, // Match string end without consuming it
        mode: cm.getMode(config, 'text/x-finesse3'),
        delimStyle: 'delim',
      }
    );
  });

  cm.defineMIME('text/x-finesse-python', 'finesse-python');

  cm.modeInfo.push({
    name: 'Finesse Python',
    mime: 'text/x-finesse-python',
    mode: 'finesse-python',
    ext: [],
  });
}
